from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.logging import LoggerABC
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.custom_file_logger_abc import CustomFileLoggerABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.abc.module_abc import ModuleABC
from bot_core.events.core_on_ready_event import CoreOnReadyEvent
from bot_core.logging.command_logger import CommandLogger
from bot_core.logging.database_logger import DatabaseLogger
from bot_core.logging.message_logger import MessageLogger
from bot_core.pipes.date_time_offset_pipe import DateTimeOffsetPipe
from bot_core.service.client_utils_service import ClientUtilsService
from bot_core.service.message_service import MessageService


class CoreModule(ModuleABC):

    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, lambda x: x.base_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        # custom logging
        services.add_singleton(CustomFileLoggerABC, CommandLogger)
        services.add_singleton(CustomFileLoggerABC, DatabaseLogger)
        services.add_singleton(CustomFileLoggerABC, MessageLogger)

        services.add_transient(MessageServiceABC, MessageService)
        services.add_transient(ClientUtilsServiceABC, ClientUtilsService)

        # pipes
        services.add_transient(DateTimeOffsetPipe)

        self._dc.add_event(DiscordEventTypesEnum.on_ready.value, CoreOnReadyEvent)

        provider = services.build_service_provider()
        # instantiate custom logger
        for c in CustomFileLoggerABC.__subclasses__():
            i: LoggerABC = provider.get_service(c)
